#ifndef __WM_MODULE_INC__
#define __WM_MODULE_INC__
/*
    Bennu Convenience Lib Set.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

#include "../lib/input_module.inc.prg"
#include "../lib/text.inc.prg"

#ifndef USING_FENIX
import "mod_map"
import "mod_grproc"
import "mod_say"

import "mod_screen"
import "mod_draw"
#endif

const
  WM_W_SELECTED = 1;
  WM_W_ACTIVE   = 2;
  WM_W_TOOLBAR  = 4;
  WM_W_ICONIZED = 8;  
  WM_W_UPDCONT  = 16; // content updated
  WM_W_HEADLESS = 32;
  WM_W_SELFPLACED = 64; // do not use autoplacement
  WM_W_UNREGISTRED = 128; // do not register window == window cannot be selected
  WM_W_SOFTAPPEAR  = 256; // appears using alpha 0->255
  
  WM_MAX_WINDOWS = 30;
end

global
  int wm_size       = 10; // basic skin size, = window's header height
  int wm_pic_bar    = 0;  // default window border
  int wm_pic_hl_bar = 0;  // bar for header length
  int wm_pic_bg_bar = 0;  // default background
  int wm_pic_corner = 0;  // top-right corner element (the same used for bottom-left and toolbar tabs)
  int wm_pic_arrow  = 0;  // "arrow" element in window header
  int wm_pic_iconhl = 0;  // frame around highlidhted icon in the bottom toolbar
  int wm_pic_icon_s = 0;  // window status icon "selected"
  int wm_pic_icon_a = 0;  // window status icon "activated"
  
  int wm_pic_mouse = 0;   // pointer
  
  int wm_c_fg   = 0; // skin color: lines
  int wm_c_bg   = 0; // skin color: window background
  int wm_c_desk = 0; // skin color: black
  
  wm_window wm_wnds[ WM_MAX_WINDOWS ];
  
  int wm_selected = -1; // selected window
  // wm_wnds = [ w,w,w,w,w,w,w,w,S,0,0,0,0,0,0,0 ]
  
  int WM_KEY_LEFT   = _JKEY_LEFT;
  int WM_KEY_RIGHT  = _JKEY_RIGHT;
  int WM_KEY_UP     = _JKEY_UP;
  int WM_KEY_DOWN   = _JKEY_DOWN;
  int WM_KEY_TAB1   = _JKEY_L;
  int WM_KEY_TAB2   = _JKEY_R;
  int WM_KEY_META1  = _JKEY_MENU;
  int WM_KEY_META2  = _JKEY_SELECT;
  int WM_KEY_OK     = _JKEY_A;
  int WM_KEY_CANCEL = _JKEY_B;
  int WM_KEY_HEAD   = _JKEY_Y;
  int WM_KEY_BOTT   = _JKEY_X;
  int WM_KEY_SYS1   = _JKEY_VOLUP;
  int WM_KEY_SYS2   = _JKEY_VOLDOWN;
  
  int WM_FEATURE_TOUCHSCREEN = 0;
  
  int wm_z_inact = 100 + WM_MAX_WINDOWS;
  int wm_z_act  = 10;
end;

/*--------------------------------------------------------------------------*/

declare process wm_window(int content_process, int wf)
public
  string title  = "";
  string bottom = "";
  int fw,fh,dx,dy;
  int wflags, wf_ex;
  int ww, wh; /// win width x height
end
end;

declare process wm_manager()
public
  int i,j, xa, ya;
  int mox, moy, mdx, mdy, mpress, mrelcnt, mwin; /// mouse follow
end
end;

/*--------------------------------------------------------------------------*/

function wm_init()
local int i; end
begin
  for ( i=0; i<WM_MAX_WINDOWS ; i++ )
    wm_wnds[ i ] = 0;
  end
end

/*--------------------------------------------------------------------------*/

function wm_selcut( int idx )
local int i,j; end
begin
  for ( i=idx; i<WM_MAX_WINDOWS-1 && wm_wnds[ i ]; i++ )
    wm_wnds[ i ] = wm_wnds[ i+1 ];
  end
end

/*--------------------------------------------------------------------------*/

function wm_setflags( )
local int i; end
begin
  for ( i=0; i<wm_selected && wm_wnds[ i ]; i++ )
      wm_wnds[ i ].wflags &= ~(WM_W_SELECTED | WM_W_ACTIVE);
      wm_wnds[ i ].z = wm_z_inact - i;
  end
  if( wm_selected>=0 )
      wm_wnds[ wm_selected ].wflags |= WM_W_SELECTED;
      wm_wnds[ wm_selected ].z = wm_z_act;
  end
end

/*--------------------------------------------------------------------------*/

function wm_window_clicked( )
local int i; end
begin
  for ( i=wm_selected; i>=0 && wm_wnds[ i ]; i-- )
      if( wm_wnds[ i ].x < mouse.x &&  wm_wnds[ i ].y < mouse.y &&
	wm_wnds[ i ].x + wm_wnds[ i ].ww > mouse.x &&  wm_wnds[ i ].y + wm_wnds[ i ].wh > mouse.y )
	return wm_wnds[ i ];
      end;
  end
  return 0;
end

/*--------------------------------------------------------------------------*/

function wm_select( int wproc )
local int i,j; end
begin
  // find wproc
  for( i=0; i<=wm_selected; i++ )
    if( wm_wnds[ i ] == wproc ) break; end;
  end
  if( i <= wm_selected)
    // remove wproc
    wm_selcut( i );
    // put wproc at the end
    //wm_selected = i;
    wm_wnds[ wm_selected ] = wproc;
  end
  wm_setflags();
end;

/*--------------------------------------------------------------------------*/

function wm_select_next()
local int i,j; end
begin
  if(wm_selected>0)
    j = wm_wnds[ 0 ];
    wm_selcut( 0 );
    wm_wnds[ wm_selected ] = j;    
    wm_setflags();
  end;
end;

/*--------------------------------------------------------------------------*/

function wm_select_prev()
local int i,j; end
begin
  if(wm_selected>0)
    i = wm_wnds[ wm_selected ];
    j = wm_wnds[ wm_selected-1 ];
    wm_wnds[ wm_selected ] = j;
    wm_wnds[ wm_selected-1 ] = i;
/*    for( i=wm_selected; i>0; i-- ) wm_wnds[ i ] = wm_wnds[ i-1 ]; end;
    wm_wnds[ 0 ] = j;  */
    wm_setflags();
  end;
end;

/*--------------------------------------------------------------------------*/

function wm_register_win(int idx)
local int i; end
begin
  if( wm_selected < WM_MAX_WINDOWS -1 )
      wm_selected++;
      if( wm_selected == 0 )
	wm_wnds[ wm_selected ] = idx;
      else
	wm_wnds[ wm_selected ] = wm_wnds[ wm_selected - 1 ];
	wm_wnds[ wm_selected - 1 ] = idx;
      end
      wm_setflags();
  end;
end;

/*--------------------------------------------------------------------------*/

function wm_free_win(int idx)
local int i; end
begin
  for ( i=0; i<WM_MAX_WINDOWS; i++ )
    if ( wm_wnds[ i ] == idx ) 
      say("Window released in slot "+i);
      wm_selcut( i ); 
      wm_selected--;
      wm_setflags();
      return;
    end;
  end;
end;

/*--------------------------------------------------------------------------*/
/***
***/
process wm_busy_space(int graph)
begin 
x = y = 0;
loop frame; end;
end;

process wm_pos_searcher(int graph, int bg, int *finalx, int *finaly)
 local int w, h, sw, sh; end
begin
  x  = y = 1;
  sw = graphic_info( 0, 0, G_WIDTH );
  sh = graphic_info( 0, 0, G_HEIGHT );
  w  = sw / 10;
  h  = w;
  while ( collision( bg ) )
    x += w/2 +1;
    if( x + w > sw) 
      x = 1;
      y += h/2 +1;
      if( y + h > sh) x=1; y=1; break; end;
    end;
    //frame;
  end;
  *finalx = x;
  *finaly = y;
end;

function wm_optimal_pos( int winproc )
local 
  int sw, sh, wsr, wsp, px, py;
  int busy, i, j;
end
begin
  px = 0; py = 0;
  if( wm_selected >= 0)
    sw = graphic_info( 0, 0, G_WIDTH );
    sh = graphic_info( 0, 0, G_HEIGHT );
    busy = map_new( sw, sh, 16);
    
    /// we will try place window NOT covering
    /// last 10 open windows;
    for( i=wm_selected, j=0; i>=0 && j<10 ; i--,j++)
      if( wm_wnds[ i ] !=winproc )
	map_put( file, busy, wm_wnds[ i ].graph, wm_wnds[ i ].x, wm_wnds[ i ].y);
      end;
    end;
    set_center(file, busy, 0, 0);
    
//     map_clear( 0, 0, rgb(0,0,0) );
//     map_xput( 0, 0, busy, 0, 0, 0, 100, 4);
    
    wsp = wm_busy_space( busy );
    wsr = wm_pos_searcher( winproc.graph, wsp, &px, &py);
    while( exists(wsr) ) frame; end;
    signal(wsp, s_kill); frame;
    map_unload( file, busy );
  end;
  winproc.x = px;
  winproc.y = py;
end

/*--------------------------------------------------------------------------*/

function wm_load_skin(string pngname)
local
  int pic, p , w;
end
begin
  pic = png_load(pngname);
  if (!pic)
    say("Cannot load skin");
    return;
  end;
  
  wm_size = graphic_info ( file, pic, G_HEIGHT );
  w = graphic_info ( file, pic, G_WIDTH );
  
  // геометрия отфанарная. привет испанцам
  wm_pic_bar = map_new( 1, wm_size+1, 16 );
  point_set ( file, wm_pic_bar, 0, 0, 0 );
  wm_pic_hl_bar = map_new( 1, wm_size+1, 16 );
  point_set ( file, wm_pic_hl_bar, 0, 0, 0 );
  wm_pic_bg_bar = map_new( 1, wm_size+1, 16 );
  point_set ( file, wm_pic_bg_bar, 0, 0, 0 );
  wm_pic_corner = map_new( wm_size, wm_size, 16 );
  point_set ( file, wm_pic_corner, 0, 0, 0 );
  wm_pic_arrow  = map_new( 9, wm_size, 16 );
  point_set ( file, wm_pic_arrow, 0, 0, 0 );
  wm_pic_iconhl = map_new( wm_size+10, wm_size, 16 );
  point_set ( file, wm_pic_iconhl, 0, 0, 0 );
  wm_pic_icon_s = map_new( 13, wm_size, 16 );
  point_set ( file, wm_pic_icon_s, 0, 0, 0 );
  wm_pic_icon_a = map_new( 13, wm_size, 16 );
  point_set ( file, wm_pic_icon_a, 0, 0, 0 );
  wm_pic_mouse = map_new( wm_size, wm_size, 16 );
  point_set ( file, wm_pic_mouse, 0, 0, 0 );
  
  p = 0; 
  map_block_copy ( file, wm_pic_bar     ,0, 0, pic , p++, 0, 1, wm_size, 0);
  p += 1; 
  map_block_copy ( file, wm_pic_hl_bar  ,0, 0, pic , p++, 0, 1, wm_size, 0);
  p += 1; 
  map_block_copy ( file, wm_pic_bg_bar  ,0, 0, pic , p++, 0, 1, wm_size, 0);
  p += 1; 
  map_block_copy ( file, wm_pic_corner  ,0, 0, pic , p++, 0, wm_size, wm_size, 0);
  p += wm_size; 
  map_block_copy ( file, wm_pic_iconhl  ,0, 0, pic , p++, 0, wm_size+10, wm_size, 0);
  p += wm_size+10; 
  map_block_copy ( file, wm_pic_arrow   ,0, 0, pic , p++, 0, 9, wm_size, 0);
  p += 9; 
  map_block_copy ( file, wm_pic_icon_a  ,0, 0, pic , p++, 0, 13, wm_size, 0);
  p += 13; 
  map_block_copy ( file, wm_pic_icon_s  ,0, 0, pic , p++, 0, 13, wm_size, 0);
  p += 13;   
  
  map_block_copy ( file, wm_pic_mouse  ,0, 0, pic , w - wm_size, 0, wm_size, wm_size, 0);
  
  wm_c_bg   = map_get_pixel( file, wm_pic_bg_bar, 0, 0);
  wm_c_fg   = map_get_pixel( file, wm_pic_bar, 0, 0);
  wm_c_desk = rgb(0,0,0);
  
  map_unload(file, pic);
  say("Skin loaded, size: "+wm_size);
end

/*--------------------------------------------------------------------------*/

function wm_free_skin()
begin
  if ( wm_pic_bar )    map_unload ( file, wm_pic_bar ); end;
  if ( wm_pic_hl_bar ) map_unload ( file, wm_pic_hl_bar ); end;
  if ( wm_pic_bg_bar ) map_unload ( file, wm_pic_bg_bar ); end;
  if ( wm_pic_corner ) map_unload ( file, wm_pic_corner ); end;
  if ( wm_pic_arrow )  map_unload ( file, wm_pic_arrow ); end;
  if ( wm_pic_iconhl ) map_unload ( file, wm_pic_iconhl ); end;
  if ( wm_pic_icon_s ) map_unload ( file, wm_pic_icon_s ); end;
  if ( wm_pic_icon_a ) map_unload ( file, wm_pic_icon_a ); end;
  if ( wm_pic_mouse ) map_unload ( file, wm_pic_mouse ); end;
end;

/*--------------------------------------------------------------------------*/
// actual skin implementation
// 
function mw_outline_frame(int wd, int hg, int winflags, string title, string bott)
local
  int swd, shg, i, j, txt;
  int cbg, cfg;
  int res;
  int hsz;
end
begin
  cbg = wm_c_bg;
  cfg = wm_c_fg;
  
  // iconized
  if( winflags & WM_W_ICONIZED )
    res = map_new( wm_size *2, wm_size + wm_size/2, 16 );
    //map_clear(file, res, rgb(255,0,0));
    point_set ( file, res, 0, 0, 0 );
    /// Toolbar
    map_xputnp ( file, res, file , wm_pic_bar,    0,         wm_size/2, 0, 100, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_hl_bar, 1,         wm_size/2, 0, wm_size*200, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_corner,  wm_size *2-1, wm_size + wm_size/2-1, 0, 100, 100, B_HMIRROR | B_VMIRROR);
    /// window header
    map_xputnp ( file, res, file , wm_pic_corner, 0,           0,         0, 100, 100, 0 );
    map_xputnp ( file, res, file , wm_pic_arrow,  wm_size + 8, 0, 0, 100, 100, B_HMIRROR  ); 
    map_xputnp ( file, res, file , wm_pic_hl_bar, wm_size + 9, 0,         0, (wm_size*2-9)*200, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_bar,    wm_size*2-1, 0,         0, 100, 100, 0 ); 

    if ( winflags & WM_W_SELECTED )
      map_xputnp ( file, res, file , wm_pic_icon_s, wm_size/2-2, 0, 0, 100, 100, 0 ); 
    end;
    
    if ( winflags & WM_W_ACTIVE )
      map_xputnp ( file, res, file , wm_pic_icon_a, wm_size/2-2, 0, 0, 100, 100, 0 ); 
    end;    
    return res;
  end;
  
  // not iconized  
  if( wd < wm_size*2 ) wd = wm_size*2; end;
  
  // 1 - window has bottom tolbar  
  if ( winflags & WM_W_TOOLBAR )
    res = map_new( wd + 2, hg + wm_size*2, 16 );
  else
    res = map_new( wd + 2, hg + wm_size+1, 16 );
  end;
  point_set ( file, res, 0, 0, 0 );
  
  drawing_map ( file, res );
  drawing_alpha ( 255 );
  
  // scales:
  swd = wd * 100 / wm_size;
  shg = hg * 100 / wm_size;
//   say(wd  + "x" + hg);
//   say(swd + ":" + shg);
  
  /// central fill
  drawing_color ( cbg );
  draw_box( 0, wm_size, wd, hg + wm_size );
  
  
  if( ! (winflags & WM_W_HEADLESS) )
    set_center( file, res, 0, 0);
    /// vertical lines
    drawing_color ( cfg );
    draw_line( 0, wm_size, 0, hg + wm_size-1 );
    draw_line( wd+1, wm_size, wd+1, hg + wm_size-1 );
    /// window header
    map_xputnp ( file, res, file , wm_pic_corner, 0,            0, 0, 100, 100, 0 );
    // WTF SCALE??? BUG?
    map_xputnp ( file, res, file , wm_pic_hl_bar, wm_size,      0, 0, (wd-wm_size*2-9)*200, 100, 0 );
    map_xputnp ( file, res, file , wm_pic_arrow,  wd-9-wm_size, 0, 0, 100, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_hl_bar, wd-wm_size,   0, 0, wm_size*200, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_bar,    wd+1,         0, 0, 100, 100, 0 ); 
    if( title != "" )
      set_text_color( cfg );
      txt = text_to_map( 0, title );
      map_put ( file, res, txt, wm_size/2, wm_size/2 - graphic_info( file, txt, G_HEIGHT )/2 );
    end
  else
    set_center( file, res, 0, 0);
    drawing_color ( cfg );
    // horiz line
    draw_line( 0, wm_size-1, wd, wm_size-1);
    /// vertical lines
    draw_line( 0, wm_size, 0, hg + wm_size-1 );
    draw_line( wd+1, wm_size, wd+1, hg + wm_size-1 );
    draw_line( 0, wm_size, wd+2, wm_size);
    set_center( file, res, 0, wm_size-1);
  end
  
  if( ! (winflags & WM_W_HEADLESS) )
    if ( winflags & WM_W_SELECTED )
      map_xputnp ( file, res, file , wm_pic_icon_s, wd-wm_size,   0, 0, 100, 100, 0 ); 
    end;
    
    if ( winflags & WM_W_ACTIVE )
      map_xputnp ( file, res, file , wm_pic_icon_a, wd-wm_size,   0, 0, 100, 100, 0 ); 
    end;
  end
  
  /// toolbar
  if ( winflags & WM_W_TOOLBAR )
    map_xputnp ( file, res, file , wm_pic_bar,    0,    hg+wm_size,   0, 100, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_hl_bar, 1, 	wm_size+hg,   0, (wd-wm_size)*200, 100, 0 ); 
    map_xputnp ( file, res, file , wm_pic_corner, wd+1, hg+wm_size*2, 180000, 100, 100, 0);
    if( bott != "" )
      txt = text_to_map( 0, bott );
      map_put ( file, res, txt, wm_size/2, hg+wm_size*1.5 - graphic_info( file, txt, G_HEIGHT )/2 );
      map_unload( file, txt );
    end
  else
    drawing_color ( cfg );
    draw_rect( 0, hg + wm_size, wd+1, hg+wm_size );
  end;
  
  return res;
  
end;

/*--------------------------------------------------------------------------*/

process wm_window(int content_process, wf )
begin
  content_process.alpha = 0;
  frame;
  wflags = wf;
  
  if( wflags & WM_W_SOFTAPPEAR ) alpha = 0; end
  
  if( ! ( wflags & WM_W_UNREGISTRED ) )
    wm_register_win( id );
  end;
  
  set_center( file, content_process.graph, 0, 0 );
  fw = graphic_info ( file, content_process.graph, G_WIDTH ) * content_process.size /100;
  fh = graphic_info ( file, content_process.graph, G_HEIGHT )* content_process.size /100;
  dx = graphic_info ( file, content_process.graph, G_X_CENTER);
  dy = graphic_info ( file, content_process.graph, G_Y_CENTER);  
  graph = mw_outline_frame ( fw, fh, wflags, title, bottom );
  ww = graphic_info ( file, graph, G_WIDTH );
  wh = graphic_info ( file, graph, G_HEIGHT );  
  
  if( ! ( wflags & WM_W_SELFPLACED ) )
    wm_optimal_pos( id );
  end;
  
  if( ! ( wflags & WM_W_ICONIZED ) )
    map_xput( file, graph, content_process.graph, dx + 1, dy + wm_size, 0, content_process.size, 0 );
  end;
  
  content_process.alpha = 0;
  content_process.x = 0;
  content_process.y = 0;
  
  if( wflags & WM_W_ACTIVE )
    input_allow( content_process );
  end  
  
  
  loop    
  
    if( wflags & WM_W_SOFTAPPEAR && alpha < 255) 
      alpha += 15;
      frame; 
    end;
  
    wf_ex = wflags;
        
    frame;
    
    if( ! exists( content_process ) ) return; end;
    
    if ( wf_ex != wflags )
      fw = graphic_info ( file, content_process.graph, G_WIDTH ) * content_process.size /100;
      fh = graphic_info ( file, content_process.graph, G_HEIGHT )* content_process.size /100;
      map_unload(file, graph);
      graph = mw_outline_frame ( fw, fh, wflags, title, bottom );
      ww = graphic_info ( file, graph, G_WIDTH );
      wh = graphic_info ( file, graph, G_HEIGHT );  
    
      if( ! ( wflags & WM_W_ICONIZED ) )
	map_xput( file, graph, content_process.graph, dx + 1, dy + wm_size, 0, content_process.size, 0);
      end;
    
      if( wflags & WM_W_ACTIVE )
	input_allow( content_process );
      else
	//input_allow( 0 );
      end            
      wflags &= ~WM_W_UPDCONT;
      
    end;
       
    if( ! ( wflags & WM_W_ICONIZED )  && ( content_process.x || content_process.y ) )
      fw = graphic_info ( file, content_process.graph, G_WIDTH ) * content_process.size /100;
      fh = graphic_info ( file, content_process.graph, G_HEIGHT )* content_process.size /100;
      map_unload( file, graph );
      graph = mw_outline_frame ( fw, fh, wflags, title, bottom );
      ww = graphic_info ( file, graph, G_WIDTH );
      wh = graphic_info ( file, graph, G_HEIGHT );  
      map_xput( file, graph, content_process.graph, dx + 1, dy + wm_size, 0, content_process.size, 0);
      content_process.x = 0;
      content_process.y = 0;
    end;
    
  end;
onexit
  wm_free_win(id);
  if( exists( content_process ) ) signal( content_process, S_KILL); end;
  map_unload( file, graph );
end;

/*--------------------------------------------------------------------------*/

process wm_manager()
begin
  wm_init();
  
  input_grab( id );
  mouse.graph = wm_pic_mouse;
  mpress = 0;  
  mwin = 0;
  
  loop
    mox = mouse.x;
    moy = mouse.y;
    
    frame;

    if( mpress && mwin)
      if( WM_FEATURE_TOUCHSCREEN )
	if( (mouse.x-mox)>25 || (mouse.x-mox)<-25 ) mouse.x=mox; end;
	if( (mouse.y-moy)>25 || (mouse.y-moy)<-25 ) mouse.y=moy; end;
      end
      mwin.x = mouse.x + mdx;
      mwin.y = mouse.y + mdy;
    end
    
    if(mouse.left) 
      if( mpress == 0) 
	mwin = wm_window_clicked();
	if( mwin )
	  /// window clicked
	  if( ! jkey( WM_KEY_META2 ) )
	    wm_select( mwin );
	  end;
	  mdx = mwin.x - mouse.x;
	  mdy = mwin.y - mouse.y;
	end;
      end;
      mpress = 1; 
    else
      mpress = 0;
      mwin = 0;
    end;
    
    if( wm_selected >= 0)
      
      if( jkey( WM_KEY_HEAD ) )  
	wm_wnds[ wm_selected ].wflags ^= WM_W_ICONIZED; 
	while( jkey( WM_KEY_HEAD ) ) frame; end;
	continue;
      end;
        
      if( jkey( WM_KEY_META1 ) && jkey( WM_KEY_BOTT ) )  
	while( jkey( WM_KEY_META1 ) || jkey( WM_KEY_BOTT ) ) frame; end;
	signal( wm_wnds[ wm_selected ], S_KILL );
	frame;
	continue;
      end;
      
      if( jkey( WM_KEY_META2 ) )  
	wm_wnds[ wm_selected ].wflags &= ~WM_W_ACTIVE;
	input_allow( 0 );
      else
	//
	wm_wnds[ wm_selected ].wflags |=  WM_W_ACTIVE;
      end;
      
      if( ! ( wm_wnds[ wm_selected ].wflags & WM_W_ACTIVE ) )
	
	if( jkey( WM_KEY_TAB1 ) )
	  wm_select_prev();
	  while( jkey( WM_KEY_TAB1 ) ) frame; end;
	  continue;
	end;
	
	if( jkey( WM_KEY_TAB2 ) )
	  wm_select_next();
	  while( jkey( WM_KEY_TAB2 ) ) frame; end;
	  continue;
	end;
	
	if( jkey( WM_KEY_LEFT ) )  wm_wnds[ wm_selected ].x--; end;
	if( jkey( WM_KEY_RIGHT ) ) wm_wnds[ wm_selected ].x++; end;
	if( jkey( WM_KEY_UP ) )    wm_wnds[ wm_selected ].y--; end;
	if( jkey( WM_KEY_DOWN ) )  wm_wnds[ wm_selected ].y++; end;
      end;
    end;
  end;
end;

/*--------------------------------------------------------------------------*/
// a simple manager. only brings active window to top
process wm_manager_simple()
begin
  wm_init();
  
  mouse.graph = 0 ;
  input_grab( id );
  
  loop
    frame;
    if( wm_selected >= 0)
      wm_wnds[ wm_selected ].wflags |=  WM_W_ACTIVE;
    else
      wm_select_prev();
    end;
  end;
  
end;

#endif
