#ifndef __WM_UTILS_INC__
#define __WM_UTILS_INC__
/*
    Bennu Convenience Lib Set.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

#include "../lib/input_module.inc.prg"
#include "../lib/wm.inc.prg"
#include "../lib/text.inc.prg"

#ifndef USING_FENIX
import "mod_time"
import "mod_map"
#endif

/*---------------------------------------------------------------------------*/

declare function __wmu_linux26_memstat( int *total_bytes, int *free_bytes )
local
  int fd, j;
  string buf;
  string stot;
  string sfre;
end
end

/*---------------------------------------------------------------------------*/
global
  int __wmu_pre_used = 0;
  int __wmu_pre_sum  = 0;
end
declare function __wmu_linux26_cpustat( int *usage_percents )
local
  int fd, i, j, used, sum;
  string buf;
  int stats[10];
end
end

/*---------------------------------------------------------------------------*/

process wmu_sysmon()
public
  wm_window window;
end
local
  int mem_total = 0;
  int mem_free  = 0;
  
  int real_fps  = 0;
  int cpu_usage = 0;
  
  int timelast, fcnt, tx;
  string buf;
  
  int gmem, gcpu, gfps;
end
begin
  
  graph = 0;
  window = wm_window( id, WM_W_SELFPLACED | WM_W_HEADLESS | WM_W_UNREGISTRED );
  window.x = 2;
  window.y = 2;
  window.alpha = 0;
  //alpha = 0;
  
  timelast = time();
  fcnt = 0;
  gmem=gcpu=gfps=0;
  
  loop
  
    if( mem_total )
      gmem = text_to_map( 0, "MEM: "+(mem_free*100/mem_total)+"% ("+mem_free+")  ");
      gcpu = text_to_map( 0, "CPU: "+cpu_usage+"%" );
      gfps = text_to_map( 0, "FPS: "+real_fps );
      tx = graphic_info( file, gmem, G_HEIGHT );
					      //////???????????????????????
      if(! graph )
	graph = map_new( graphic_info( file, gmem, G_WIDTH ), tx*3, 16 );
	set_center( file, graph, 0, 0 );
      end
	
      map_clear( file, graph, wm_c_bg );
      map_put( file, graph, gmem, 1, 0 );
      map_put( file, graph, gcpu, 1, tx );
      map_put( file, graph, gfps, 1, tx*2 );
      map_unload( file, gmem );
      map_unload( file, gcpu );
      map_unload( file, gfps );

      window.alpha = 200;    
      x++;    
      frame 500;
    else
        frame ;
    end;
        
    /// Expects linux >=2.6, so it would provide false on GPX ( linux 2.4 )
    /// Be warned !
    __wmu_linux26_memstat( &mem_total, &mem_free );
    __wmu_linux26_cpustat( &cpu_usage );
    
    fcnt++;
    if( (time() - timelast) > 2 )
      real_fps = fcnt / ( time() - timelast );
      timelast = time();
      fcnt = 0;
    end
  end;
onexit
  map_unload( file, graph );
end

/*---------------------------------------------------------------------------*/

global

string __wmu_key_visual[] = (
    "../res/wm_utils/kk-up.png",
    "",
    "../res/wm_utils/kk-left.png",
    "",
    "../res/wm_utils/kk-down.png",
    "",
    "../res/wm_utils/kk-right.png",
    "",
    "../res/wm_utils/kk-M.png",
    "../res/wm_utils/kk-S.png",
    "../res/wm_utils/kk-L.png",
    "../res/wm_utils/kk-R.png",   
    "../res/wm_utils/kk-A.png",
    "../res/wm_utils/kk-B.png",
    "../res/wm_utils/kk-X.png",
    "../res/wm_utils/kk-Y.png",
    "../res/wm_utils/kk-volup.png",
    "../res/wm_utils/kk-voldown.png"
);

// 0   1
// align
// 3   2
int __wmu_key_g_align[] = (
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    1,
    0,
    1,   
    1,
    1,
    1,
    1,
    0,
    2
);
end

process wmu_jkey_assigner( int idev , int wflags )
public
  wm_window window;
end
local
  int __wmu_key_g[ _JKEY_LAST ];
  int new_keys[ _JKEY_LAST ];
  int i, t1, t2, ww, hh, tw,th, coler;
  int g_grid, g_marked, g_mark;
  int jcode, iev;
end
begin
  
  window = id;
  g_grid = png_load("../res/wm_utils/kk-bg.png");
  set_center ( file, g_grid, 0, 0 );
  ww = graphic_info( file, g_grid, G_WIDTH );
  hh = graphic_info( file, g_grid, G_HEIGHT );
  
  // convert to 16 bit
  t1 = new_map( ww, hh, default_bpp );
  set_center ( file, t1, 0, 0 );
  map_put( file, t1, g_grid, 0, 0 );
  map_unload( file, g_grid );
  g_grid = t1;
  
  coler = new_map( ww, hh, default_bpp );
  map_clear ( file, coler, wm_c_fg );
  set_center ( file, coler, 0, 0 );
  
  // colorize
  map_xput( file, g_grid, coler, 0, 0, 0, 100, B_SBLEND );
  
  g_marked = map_clone( file, g_grid );  
  
  for( i=0; i< _JKEY_LAST; i++)
    if( im_key_configurable[ i ] )
      __wmu_key_g[ i ] = png_load( __wmu_key_visual[ i ] );
      tw = graphic_info( file, __wmu_key_g[ i ], G_WIDTH );
      th = graphic_info( file, __wmu_key_g[ i ], G_HEIGHT );
    
      t1 = new_map( tw, th, 16 );
      set_center ( file, t1, 0, 0 );
      set_center ( file, __wmu_key_g[ i ], 0, 0 );    
      map_put( file, t1, __wmu_key_g[ i ], 0, 0 );
      map_unload( file, __wmu_key_g[ i ] );
      __wmu_key_g[ i ] = t1;
      map_xput( file, t1, coler, 0, 0, 0, 100, B_SBLEND );
    
      if( __wmu_key_g_align[ i ] == 0 )
	set_center( file, __wmu_key_g[ i ], 0, 0 );
      end
      if( __wmu_key_g_align[ i ] == 3 )
	set_center( file, __wmu_key_g[ i ], 0, th-hh );
      end
      if( __wmu_key_g_align[ i ] == 1 )
	set_center( file, __wmu_key_g[ i ], tw-ww, 0 );
      end
      if( __wmu_key_g_align[ i ] == 2 )
	set_center( file, __wmu_key_g[ i ], tw-ww, th-hh );
      end
    end
  end
  
  map_unload( file, coler );
  
  for( i=0; i< _JKEY_LAST; i++)
    if( im_key_configurable[ i ] )
      map_put( file, g_marked, __wmu_key_g[ i ], 0, 0 );
      new_keys[ i ] = 0;
    end
  end
  
  graph = g_marked;
  window = wm_window( id, wflags | WM_W_TOOLBAR );
  window.title  = input_dev_name( idev );
  window.bottom = "Press ANY key";

  while ( !__im_get_iev( idev ) )
    frame;
  end;
  
  window.bottom = "OK";
  x++;
  
  timer[1] = 0;
  while( timer[1] < 300)
    frame;
  end;
  
  while ( __im_get_iev( idev ) )
    frame;
  end;
  
  for( jcode = 0; jcode< _JKEY_LAST; jcode ++)
    if( im_key_configurable[ jcode ] )
      g_mark = map_clone( file, g_grid );
      set_center( file, g_mark, 0, 0 );
      map_put( file, g_mark, __wmu_key_g[ jcode ], 0, 0 );
      graph  = g_mark;
      window.bottom = "Press "+im_key_name[ jcode ];
      x++;
      while ( !( iev = __im_get_iev( idev ) ) )
	frame;
      end;
    
      t1 = 0;    
      for( i = 0; i< _JKEY_LAST; i++ )
	if( new_keys[ i ] == iev )
	  
	  window.bottom = "NO! Ambigous!";
	  x++;
	  
	  timer[1] = 0;
	  while( timer[1] < 200)
	    frame;
	  end;
	  
	  jcode--;
	  t1 = 1;
	  break;
	end
      end
      
      if( t1 ) continue; end
	
      window.bottom = "OK";
      x++;
      
      timer[1] = 0;
      while( timer[1] < 200)
	frame;
      end;
      
      while ( iev ==__im_get_iev( idev ) )
	frame;
      end;
      
       new_keys[ jcode ] = iev;
      //__im_assign_jkey2event( idev, jcode, iev );
  
	
      map_unload( file, g_mark );
    end
  end
  
  for(i = 0; i< _JKEY_LAST; i++)
      __im_assign_jkey2event( idev, i, new_keys[ i ] );    
  end
onexit
  map_unload( file, g_grid );
  map_unload( file, g_marked );
  
  for( i=0; i< _JKEY_LAST; i++)
    if( im_key_configurable[ i ] )
      map_unload( file, __wmu_key_g[ i ] );
    end
  end
end

/*---------------------------------------------------------------------------*/

function __wmu_linux26_memstat( int *total_bytes, int *free_bytes )
/*local
  int fd, j;
  string buf;
  string stot;
  string sfre;
end*/
begin
  fd = fopen("/proc/meminfo", O_READ);
  if( !fd ) return; end;
  while( !feof( fd ) )
    buf = fgets(fd);
    if( substr( buf, 0, 9 ) == "MemTotal:" ) stot = trim( substr( buf, 10 ) ); end;
    if( substr( buf, 0, 8 ) == "MemFree:" )  sfre = trim( substr( buf, 10 ) ); end;
  end
  fclose(fd);
  
  j = atoi( stot );
  if( substr( stot, -2, 2 ) == 'kB' ) j*=1024;  end
  *total_bytes = j;
  
  j = atoi( sfre );
  if( substr( sfre, -2, 2 ) == 'kB' ) j*=1024;  end
  *free_bytes  = j;
end

/*---------------------------------------------------------------------------*/

function __wmu_linux26_cpustat( int *usage_percents )
/*local
  int fd, i, j, used, sum;
  string buf;
  int stats[10];
end
*/
begin
  fd = fopen("/proc/stat", O_READ);
  if( !fd ) return; end;
  buf = fgets(fd);
  buf = trim( substr( buf, 3, strlen(buf)-3 ) ); // cut out "cpu <tab>"
  fclose(fd);

  // user, nice, system, idle, iowait, ...10  | usage = user+system / SUM

  while( j<10 )
    i = find( buf, " " );
    if(i>=0)
      stats[j] = itoa( substr( buf, 0, i) );
      //say ( j+': '+stats[j] );
      buf = substr( buf, i+1, strlen(buf)-i-1 );
    else
      stats[j] = 0;
    end
    j++;
  end
  used = stats[0] + stats[2];
  sum  = stats[0] + stats[1] + stats[2] + stats[3] + stats[4] + stats[5] + stats[6] + stats[7] + stats[8] + stats[9];

  if(sum==0)
    *usage_percents = 0;
  else
    *usage_percents = ( used - __wmu_pre_used ) * 100 /(sum - __wmu_pre_sum );
  end
  __wmu_pre_used = used;
  __wmu_pre_sum  = sum;
end

#endif