/*
    AGGRESSIVE PONG.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

import "mod_mem"
import "mod_say"

/// animation from file.
// limitation: a frame must be rectangle.
type ani_file
  int count = 0;
  int *frames;
  int fw, fh; // frame width, frame height
end

// running animation
type ani_loop
  ani_file *ani;
  int fr = 0; // current frame;
  int ani_end = 0; // flag of animation end
end

/*-----------------------------------------------------------------------------*/
// separator = 1 if frames in file are separated with 1 pixel border
function ani_load( ani_file *ani, string filename, int separator)
local
  int ps, sw, sh, sx, cnt;
end
begin
  ani.count =0;
  ps = png_load( filename );
  if( ps )
    sw = graphic_info ( file, ps, G_WIDTH );
    sh = graphic_info ( file, ps, G_HEIGHT );
    sx = 0;
    ani.frames = alloc( sizeof(int) * ( sw/(sh+separator)+1 ) );
    cnt = 0;
    while( sx < sw )
      ani.frames[ cnt ] = map_new( sh, sh, 16 );
      map_block_copy( file, ani.frames[ cnt ], 0, 0,  ps, sx, 0, sh, sh, 0 );
      cnt++;
      sx += sh + separator;
    end
    ani.count = cnt;
    ani.fw = sh;
    ani.fh = sh;
    map_unload( file, ps );
    say('Loaded animation '+filename+', '+cnt+' frames');
  else
    say('Error loading '+filename );
  end
end

/*-----------------------------------------------------------------------------*/

function ani_unload( ani_file *ani )
 local int i; end
begin
  if( ani.count>0 )
    for( i=0; i<ani.count; i++) map_unload( file, ani.frames[ i ] ); end
    ani.count = 0;
    free( ani.frames );
  end;
end

/*-----------------------------------------------------------------------------*/

function ani_set_center( ani_file *ani, int mx int my )
 local int i; end
begin
  if( ani.count>0 )
    for( i=0; i<ani.count; i++) 
      set_center( file, ani.frames[ i ], mx, my ); 
    end
  end;
end

/*-----------------------------------------------------------------------------*/

function ani_new_loop( ani_loop *lp, ani_file *ani )
begin
  lp.ani = ani;
  lp.fr = -1;
  lp.ani_end = 0;
  if(ani.count==0) say('WARN: empyt animation'); end
end

/*-----------------------------------------------------------------------------*/

function int ani_next_frame( ani_loop *lp )
begin
  lp.fr ++;
  if( lp.fr>= lp.ani.count ) lp.fr = 0; end
  lp.ani_end = lp.fr == lp.ani.count-1;
  return lp.ani.frames[ lp.fr ];
end

/*-----------------------------------------------------------------------------*/
