/*
    AGGRESSIVE PONG.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

#include "../lib/input_module.inc.prg"
#include "../lib/wm.inc.prg"
#include "../lib/menu.inc.prg"
#include "../lib/list.inc.prg"

#define GMM_MODE_DEMO      0
#define GMM_MODE_STORY     1
#define GMM_MODE_1P        2
#define GMM_MODE_2P        3
#define GMM_MODE_OPTIONS   4
#define GMM_MODE_EXIT      5
#define GMM_MODE_CONTINUE  6

global
int gamemenu_mode = 0;
end

declare process gm_input_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  wmu_jkey_assigner ka;
  int i, t1, t2, t3;
end
end

declare process gm_selectlocale_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  int i;
end
end

declare process gm_music_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  wmu_jkey_assigner ka;
  int i, t1, t2, t3;
end
end

declare process gm_help( )
local
  int i;
end
end

declare process gm_options_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  int t1, t2, t3, redraw;
end
end

declare process gm_about_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_testbox box;
end
end

declare process gm_idev_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  int i;
end
end

////////////////////////////////////////////////////////////////

process gm_main_menu()
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  int submenu;
end
begin
  list_create( &lst );
  list_grow( &lst, _(" Story Mode ") ); // 0
  list_grow( &lst, _(" Show Demo ") );  // 1
  list_grow( &lst, _(" 1P vs CPU ") );  // 2
  if( !os_handheld ) list_grow( &lst, _(" 1P vs 2P ") ); end;   // 3
  list_grow( &lst, _(" Options ") );    // 4
  list_grow( &lst, _(" Help ") );    // 5
  list_grow( &lst, _(" About ") );    // 6
  list_grow( &lst, _(" Exit ") );       // 7
  box = menu_selectbox( lst, 10 );
  wnd = wm_window( box, WM_W_SELFPLACED | WM_W_TOOLBAR | WM_W_SOFTAPPEAR );
  wnd.title=_("MENU");
  wnd.bottom = game_version;
  frame;
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-1;
  gamemenu_mode = -1;
  wm_select(wnd);
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      //if(os_handheld && box.selected > 2 ) box.selected += 1; end
      switch( (os_handheld && box.selected > 2) ? box.selected+1 : box.selected )
	case 1 :
	  gamemenu_mode = GMM_MODE_DEMO;
	  return;
	end
	case 0 :
	  gamemenu_mode = GMM_MODE_STORY;
	  return;
	end
	case 2 :
	  gamemenu_mode = GMM_MODE_1P;
	  return;
	end
	case 3 :
	  gamemenu_mode = GMM_MODE_2P;
	  return;
	end
	case 4 :
	  gamemenu_mode = GMM_MODE_OPTIONS;
	  submenu = gm_options_menu();
	  while( exists(submenu) ) frame; end
	  if( gamemenu_mode == GMM_MODE_EXIT )
	    return;
	  end
	end
	case 5 :
	  submenu = gm_help();
	  while( exists(submenu) ) frame; end
	end
	case 6 :
	  submenu = gm_about_menu();
	  while( exists(submenu) ) frame; end
	end
	case 7 :
	  let_me_alone(); frame; exit('(X__X)');
	end
      end
    end
  end;
onexit
  input_allow(0);
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_ingame_menu()
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
end
begin
  list_create( &lst );
  list_grow( &lst, _(" Continue ") );  // 0
  list_grow( &lst, _(" Exit to Menu ") ); // 1
  box = menu_selectbox( lst, 10 );
  wnd = wm_window( box, WM_W_SELFPLACED   );
  wnd.title=_("PAUSE");
  frame ;
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = 0;

  gamemenu_mode = -1;
  while( jkey( _JKEY_MENU ) ) frame; end;
    
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_MENU )  )
      while( jkey( _JKEY_MENU ) ) frame; end;
	gamemenu_mode = GMM_MODE_CONTINUE;
	return;
    end;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      switch( box.selected )
	case 0 :
	  gamemenu_mode = GMM_MODE_CONTINUE;
	  return;
	end
	case 1 :
	  gamemenu_mode = GMM_MODE_EXIT;
	  return;
	end
      end
    end
  end;
onexit
  input_allow(0);
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_endlevel_menu( string title )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
end
begin
  list_create( &lst );
  //list_grow( &lst, _(" Play Again ") );  // 0
  list_grow( &lst, title ); // 1
  box = menu_selectbox( lst, 10 );
  wnd = wm_window( box, WM_W_SELFPLACED | WM_W_TOOLBAR  );
  wnd.title  = _("IT'S OVER");
  wnd.bottom =  _("Press a key");
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-1;
  
  gamemenu_mode = -1;
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      switch( box.selected )
	case 0 :
	  gamemenu_mode = GMM_MODE_EXIT;
	  return;
	end
// 	case 1 :
// 	  gamemenu_mode = GMM_MODE_EXIT;
// 	  return;
// 	end
      end
    end
  end;
onexit
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_options_menu( )
begin
  redraw = 1;
  list_create( &lst );
  
      list_grow( &lst, _(" Sound: ")       +(option_sound_on?'ON':'OFF') ); // 0
      list_grow( &lst, _(" Music theme: ") +(sfx_music_schemes[sfx_music_scheme])+" " ); // 1
      list_grow( &lst, _(" Blood: ")       +(option_blood_on?'ON':'OFF') ); // 2
      list_grow( &lst, _(" Startup splash: ")+(option_gspainsplash_on?'ON':'OFF') ); // 3
      list_grow( &lst, _(" System monitor: ")+(exists(sys_mon)?'ON':'OFF') ); // 4
      if( !os_handheld )
	list_grow( &lst, _(" Full Screen: ")  +(option_fulscreen_on?'ON':'OFF') ); // 5
	list_grow( &lst, _(" Define P1 Keys ") ); // 6
	list_grow( &lst, _(" Define P2 Keys ") ); // 7
	list_grow( &lst, _(" Input device config ") ); // 8
      end
      list_grow( &lst, _(" Language ") ); // 9
      list_grow( &lst, _(" Exit to Menu ") ); // 10
      
  box = menu_selectbox( lst, 12 );  
  wnd = wm_window( box,  WM_W_SELFPLACED  );
  wnd.title  = _("OPTIONS");
  frame ;
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-2;
  say('z='+wnd.z);
  wm_select(wnd);
  
  gamemenu_mode = -1;
  loop
    if( redraw )
      redraw = 0;
      list_free( &lst );
      list_create( &lst );
      list_grow( &lst, _(" Sound: ")       +(option_sound_on?'ON':'OFF') ); // 0
      list_grow( &lst, _(" Music theme: ") +(sfx_music_schemes[sfx_music_scheme])+" " ); // 1
      list_grow( &lst, _(" Blood: ")       +(option_blood_on?'ON':'OFF') ); // 2
      list_grow( &lst, _(" Startup splash: ")+(option_gspainsplash_on?'ON':'OFF') ); // 3
      list_grow( &lst, _(" System monitor: ")+(exists(sys_mon)?'ON':'OFF') ); // 4
      if( !os_handheld )
	list_grow( &lst, _(" Full Screen: ")  +(option_fulscreen_on?'ON':'OFF') ); // 5
	list_grow( &lst, _(" Define P1 Keys ") ); // 6
	list_grow( &lst, _(" Define P2 Keys ") ); // 7
	list_grow( &lst, _(" Input device config ") ); // 8
      end
      list_grow( &lst, _(" Language ") ); // 9
      list_grow( &lst, _(" Exit to Menu ") ); // 10
      box.redraw = 1;
    end;
//     input_grab(id);
    input_allow(box);
    frame;    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      //if(os_handheld && box.selected > 4 ) box.selected += 4; end
      switch( (os_handheld && box.selected > 4 ) ? box.selected+4 : box.selected )
	case 0 :
	  option_sound_on = ! option_sound_on;
	  sfx_sound_on( option_sound_on );
	  redraw = 1;
	end
	case 1 :
	  t1 = gm_music_menu( );
	  while( exists(t1) ) frame; end
	  redraw = 1;
	end
	case 2 :
	  option_blood_on = ! option_blood_on;
	  redraw = 1;
	  // return;
	end
	case 3 :
	  option_gspainsplash_on = ! option_gspainsplash_on;
	  redraw = 1;
	  // return;
	end
	case 4 :
	  if( exists(sys_mon) )
	    signal( sys_mon, S_KILL );
	  else
	    sys_mon = wmu_sysmon();
	  end
	  redraw = 1;
	  //return;
	end
	case 5 :
	  option_fulscreen_on = ! option_fulscreen_on;
	  select_mode( option_fulscreen_on );
	  redraw = 1;
	  //return;
	end
	case 6 :
	  t1 = gm_input_bind_menu( 1 );
	  while( exists(t1) ) frame; end
	end
	case 7 :
	  t1 = gm_input_bind_menu( 2 );
	  while( exists(t1) ) frame; end
	end
	case 8 :
	  t1 = gm_input_menu( );
	  while( exists(t1) ) frame; end
	end
	case 9 :
	  t1 = gm_selectlocale_menu( );
	  while( exists(t1) ) frame; end
	  gamemenu_mode = GMM_MODE_EXIT;
	  return;
	end
	case 10 :
	  gamemenu_mode = GMM_MODE_EXIT;
	  return;
	end
      end
    end
  end;
onexit
  save_options( options_file );
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_gamecomplete_menu( )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
end
begin
  list_create( &lst );
  list_grow( &lst, _(" Congratulations! Game complete! ") );  // 0
  box = menu_selectbox( lst, 10 );
  wnd = wm_window( box, WM_W_SELFPLACED | WM_W_TOOLBAR  );
  wnd.title  = _("GAME COMPLETE");
  wnd.bottom =  "";
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-1;
  
  gamemenu_mode = -1;
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      switch( box.selected )
	case 0 :
	  gamemenu_mode = GMM_MODE_EXIT;
	  return;
	end
      end
    end
  end;
onexit
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_selectlevel_menu( int maxlevel )
local
  wm_window wnd;
  stringlist lst;
  menu_selectbox box;
  int i;
end
begin
  list_create( &lst );
  list_grow( &lst, _("    Back to menu    ") );
  for( i=1; i<= maxlevel; i++ ) 
    if( i > game_max_level )
      list_grow( &lst, _("   Show me my CUP   ") );
      break;
    end
    list_grow( &lst, _("    From LEVEL ")+i+"    " );
  end
  box = menu_selectbox( lst, 12 );
  wnd = wm_window( box, WM_W_SELFPLACED | WM_W_TOOLBAR  );
  wnd.title  = _("CONTINUE ?");
  wnd.bottom = "";
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-1;
  
  gamemenu_mode = -1;
  
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      gamemenu_mode = box.selected;
      break;
    end
  end;
onexit
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_selectlocale_menu( )
begin
  list_create( &lst );
  list_grow( &lst, _(" English ") );
  list_grow( &lst, _(" Russian ") );
  list_grow( &lst, _(" Do not change ") );
  box = menu_selectbox( lst, 5 );
  wnd = wm_window( box, WM_W_SELFPLACED );
  wnd.title  = _("LANGUAGE");
  wnd.bottom = "";
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-1;
  
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
	if( box.selected == 0 ) translation_load( option_locale ="../data/messages.eng.txt"); save_options( options_file ); end;
	if( box.selected == 1 ) translation_load( option_locale ="../data/messages.rus.txt"); save_options( options_file ); end;
      break;
    end
  end;
onexit
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_idev_menu( )
begin
  list_create( &lst );
  for( i=0; i<= INPUT_MAX_INPUTDEVS && input_dev_name(i)!="" ; i++ ) 
    list_grow( &lst, _("Change to: ")+input_dev_name(i) );
  end
  list_grow( &lst, _("Keep as it is") );
  box = menu_selectbox( lst, 10 );
  wnd = wm_window( box, WM_W_SELFPLACED );
  wnd.title  = _("DEVICE");
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-5;
  
  gamemenu_mode = -1;
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      if( input_dev_name(box.selected) !="" )
	gamemenu_mode = box.selected;
      else
	gamemenu_mode = -1;
      end
      break;
    end
  end;
onexit
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_input_menu( )
begin
  list_create( &lst );
  list_grow( &lst, _("Change device, Player 1: ")+input_dev_name(p1_input) ); // 0
  list_grow( &lst, _("Change device, Player 2: ")+input_dev_name(p2_input) ); // 1
  t1 = 0;
  for( i=0; i<= INPUT_MAX_INPUTDEVS && input_dev_name(i)!="" ; i++ ) 
    t1++;
    input_set_dev(i);
    t2 = input_is_sane();
    list_grow( &lst, _("Reassign keys: ")+input_dev_name(i)+(t2?' [ok] ':' [misconfigured] ') );
  end
  input_set_dev( p1_input );
  list_grow( &lst, _(" Go back ") ); // 1+t1;
  box = menu_selectbox( lst, 10 );
  wnd = wm_window( box, WM_W_SELFPLACED );
  wnd.title  = _("INPUT");
  frame ;
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-5;
  
  gamemenu_mode = -1;
  loop
    input_allow(box);
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      
      if( box.selected == 0 )
	  input_allow(-1);
	  t3 = gm_idev_menu( );
	  while( exists(t3) ) frame; end
	    if( gamemenu_mode>=0 )
	      p1_input = gamemenu_mode;
	      input_set_dev(p1_input);
	      if( !input_is_sane() ) input_preset_safe(); end
	      gamemenu_mode = GMM_MODE_EXIT;
	      return;
	    end
	  continue;
      end
      
      if( box.selected == 1)
	  input_allow(-1);
	  t3 = gm_idev_menu( );
	  while( exists(t3) ) frame; end
	    if( gamemenu_mode>=0 )
	      p2_input = gamemenu_mode;
	      input_set_dev(p2_input);
	      if( !input_is_sane() ) input_preset_safe(); end
	      input_set_dev(p1_input);
	      gamemenu_mode = GMM_MODE_EXIT;
	      return;
	    end
	  continue;
      end
      
      if( box.selected == t1 + 2 )
	  gamemenu_mode = GMM_MODE_EXIT;
	  return;
      end
	
      input_allow(-1);
      ka = wmu_jkey_assigner( box.selected-2, WM_W_SELFPLACED | WM_W_UNREGISTRED );
      ka.window.z = 0;
      while( exists(ka) ) frame; end
      gamemenu_mode = GMM_MODE_EXIT;
      return;
    end
  end;
onexit
  save_options( options_file );
  input_set_dev(p1_input);
  input_save_config( p1_devconfig );
  input_set_dev(p2_input);
  input_save_config( p2_devconfig );
  input_set_dev(p1_input);
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////

process gm_volume_menu( )
local
  wm_window wnd;
  menu_progressbar box;
  int vol, dosave;
end
begin
  box = menu_progressbar( 0, 127, &sfx_sound_volume );
  wnd = wm_window( box, WM_W_SELFPLACED | WM_W_TOOLBAR | WM_W_UNREGISTRED  );
  wnd.alpha = 0;
  wnd.title = 'SOUND';
  wnd.bottom = 'VOLUME '+sfx_sound_volume;
  frame;
  
  wnd.x = scr_width()  - wnd.ww - 2;
  wnd.y = scr_height() - wnd.wh - 2;
  wnd.z = 0;
  
  dosave = 0;
  while( exists(wnd))
    
    if( wnd.alpha>0 ) wnd.alpha-=50; end
    if( dosave && wnd.alpha<=0 ) save_options( options_file ); end
      
    if( jkey( _JKEY_VOLUP ) )
      wnd.alpha = 255;
      dosave = 1;
      sfx_change_volume(+2);
      wnd.bottom = 'VOLUME '+sfx_sound_volume;
    end
    if( jkey( _JKEY_VOLDOWN ) )
      wnd.alpha = 255;
      dosave = 1;
      sfx_change_volume(-2);
      wnd.bottom = 'VOLUME '+sfx_sound_volume;
    end
    frame 300;
  end;
onexit
  signal(wnd, S_KILL);
end

////////////////////////////////////////////////////////////////

process gm_about_menu( )
begin
  list_create_fn( &lst, _("../data/about.eng.txt") );
  box = menu_textbox( &lst, 12 );
  wnd = wm_window( box, WM_W_SELFPLACED | WM_W_TOOLBAR  );
  wnd.title = _("ABOUT");
  wnd.bottom=_("PU/DOWN to scroll");
  frame;
  
  wnd.x = (scr_width()  - wnd.ww) / 2;
  wnd.y = (scr_height() - wnd.wh) / 2;
  wnd.z = wm_z_act-1;
  
  while( exists(wnd))
    input_allow(box);
    frame;
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
      return;
    end  
  end
  
onexit
  signal(wnd, S_KILL);
  list_free( lst );
end

////////////////////////////////////////////////////////////////////

process gm_help( )
begin
  graph = png_load("../data/help-wiz.png");
  set_center( file, graph, 0, 0 );
  z = 0;
    input_allow( id );
  while ( ajkey( _JKEY_A, id ) || ajkey( _JKEY_B, id ) || ajkey( _JKEY_MENU, id ) ) frame; end;
  
  loop
    input_allow( id );
    frame;
    if( ajkey( _JKEY_A, id ) || ajkey( _JKEY_B, id ) || ajkey( _JKEY_MENU, id ) )
      while ( ajkey( _JKEY_A, id ) || ajkey( _JKEY_B, id ) || ajkey( _JKEY_MENU, id ) ) frame; end;
      break;
    end
  end
  map_unload( file, graph );
end

////////////////////////////////////////////////////////////////

process gm_music_menu( )
begin
  list_create( &lst );
  for( i=0; sfx_music_schemes[i] != "" ; i++ ) 
    list_grow( &lst, " "+sfx_music_schemes[i]+_(" theme ") );
  end
  box = menu_selectbox( lst, 10 );
  box.selected = sfx_music_scheme;
  box.redraw =1;
  wnd = wm_window( box, WM_W_SELFPLACED );
  wnd.title  = _("MUSIC");
  frame ;
  
  wnd.x = ( scr_width()  - wnd.ww ) /2;
  wnd.y = ( scr_height() - wnd.wh ) /2;
  wnd.z = wm_z_act-5;
    
  loop
    input_allow(box);
    t1 = box.selected;
    frame;
    
    if( jkey( _JKEY_A ) || jkey( _JKEY_B ) )
      while( jkey( _JKEY_A ) || jkey( _JKEY_B ) ) frame; end;
    return;
    end
    
    if(t1 != box.selected)
      sfx_music_theme( box.selected );
    end;
    
  end;
onexit
  save_options( options_file );
  signal(wnd, S_KILL);
  list_free( lst );
end