/*
    AGGRESSIVE PONG.
    Copyright (C) 2010  Eugene Popov (popov_e_n@mail.ru)

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/

// Control keys:
const
  CK_UP     = 1;
  CK_DOWN   = 2;
  CK_LEFT   = 3;
  CK_RIGHT  = 4;
  CK_MENU   = 5;
  CK_ACT_I  = 6;
  CK_ACT_D  = 7;
  CK_TURBO  = 8;
  CK_EFFECT = 9;
  CK_VOLUP  = 10;
  CK_VOLDN  = 11;
  CK_COUNT  = 12;
end

global
  // game options
  int option_fulscreen_on = 0; // mode fullscreen. there is some bug with system full_screen  
  int option_sound_on = 1;
  int option_blood_on = 1;
  int option_gspainsplash_on = 1;
  
  // id of input device
  int p1_input = 0;
  int p2_input = 0;
  
  // control bindings for P1 & P2
  int ck_p1[ CK_COUNT ];
  int ck_p2[ CK_COUNT ];
  
  string p1_devconfig = "../var/input-p1.conf";
  string p2_devconfig = "../var/input-p2.conf";
  
  string options_file = "../var/options.conf";
  
  int game_max_level = 9;  // MAX Level
  int state_unlocked = 0; // topmost unlocked level  state_unlocked > max_level = game complete
  string state_file = "../var/game.state";
  
  string option_locale = "../data/messages.eng.txt";
end

function save_options(string filename)
local int fd; end
begin
  fd = fopen(filename, O_WRITE);
  if( fd )
    fputs( fd, "locale="+option_locale );
    fputs( fd, "fullscreen_on="+option_fulscreen_on );
    fputs( fd, "sound_on="+option_sound_on );
    fputs( fd, "music_scheme="+sfx_music_scheme );
    fputs( fd, "blood_on="+option_blood_on );
    fputs( fd, "gspainsplash_on="+option_gspainsplash_on );
    fputs( fd, "sfx_sound_volume="+sfx_sound_volume);
    fputs( fd, "p1_input_dev="+p1_input);
    fputs( fd, "p2_input_dev="+p2_input);
    fputs( fd, "p1_ck_up=" + ck_p1[ CK_UP ]);
    fputs( fd, "p1_ck_down=" + ck_p1[ CK_DOWN ]);
    fputs( fd, "p1_ck_menu=" + ck_p1[ CK_MENU ]);
    fputs( fd, "p1_ck_acti=" + ck_p1[ CK_ACT_I ]);
    fputs( fd, "p1_ck_actd=" + ck_p1[ CK_ACT_D ]);
    fputs( fd, "p1_ck_turbo=" + ck_p1[ CK_TURBO ]);
    fputs( fd, "p1_ck_effect=" + ck_p1[ CK_EFFECT ]);
    fputs( fd, "p1_ck_volup=" + ck_p1[ CK_VOLUP ]);
    fputs( fd, "p1_ck_voldn=" + ck_p1[ CK_VOLDN ]);
    fputs( fd, "p2_ck_up=" + ck_p2[ CK_UP ]);
    fputs( fd, "p2_ck_down=" + ck_p2[ CK_DOWN ]);
    fputs( fd, "p2_ck_menu=" + ck_p2[ CK_MENU ]);
    fputs( fd, "p2_ck_acti=" + ck_p2[ CK_ACT_I ]);
    fputs( fd, "p2_ck_actd=" + ck_p2[ CK_ACT_D ]);
    fputs( fd, "p2_ck_turbo=" + ck_p2[ CK_TURBO ]);
    fputs( fd, "p2_ck_effect=" + ck_p2[ CK_EFFECT ]);
    fputs( fd, "p2_ck_volup=" + ck_p2[ CK_VOLUP ]);
    fputs( fd, "p2_ck_voldn=" + ck_p2[ CK_VOLDN ]);
    fclose( fd );
  end
end

function load_options(string filename)
local
  int fd, j;
  string b,n,v;
end
begin
  fd = fopen(filename, O_READ);
  if( fd )
      while( !feof(fd) )
	b = fgets(fd);
	j = find(b,"=");
	if( j>0 )
	  n = substr(b,0,j);
	  v = substr(b,j+1,len(b)-j-1);
	  say(n+' = '+v);
	  if ( n=='locale' ) option_locale = v; continue; end;
	  if ( n=='fulscreen_on' ) option_fulscreen_on = atoi(v); continue; end;
	  if ( n=='sound_on' ) option_sound_on = atoi(v); continue; end;
	  if ( n=='music_scheme' ) sfx_music_scheme = atoi(v); continue; end;
	  if ( n=='blood_on' ) option_blood_on = atoi(v); continue; end;
	  if ( n=='gspainsplash_on' ) option_gspainsplash_on = atoi(v); continue; end;
	  if ( n=='sfx_sound_volume') sfx_sound_volume = atoi(v); continue; end
	  if ( n=='p1_input_dev') p1_input = atoi(v); continue; end
	  if ( n=='p2_input_dev') p2_input = atoi(v); continue; end
	  if ( n=='p1_ck_up') ck_p1[ CK_UP ] = atoi(v); continue; end
	  if ( n=='p1_ck_down') ck_p1[ CK_DOWN ] = atoi(v); continue; end
	  if ( n=='p1_ck_menu') ck_p1[ CK_MENU ] = atoi(v); continue; end
	  if ( n=='p1_ck_acti') ck_p1[ CK_ACT_I ] = atoi(v); continue; end
	  if ( n=='p1_ck_actd') ck_p1[ CK_ACT_D ] = atoi(v); continue; end
	  if ( n=='p1_ck_turbo') ck_p1[ CK_TURBO ] = atoi(v); continue; end
	  if ( n=='p1_ck_effect') ck_p1[ CK_EFFECT ] = atoi(v); continue; end
	  if ( n=='p1_ck_volup') ck_p1[ CK_VOLUP ] = atoi(v); continue; end
	  if ( n=='p1_ck_voldn') ck_p1[ CK_VOLDN ] = atoi(v); continue; end
	  if ( n=='p2_ck_up') ck_p2[ CK_UP ] = atoi(v); continue; end
	  if ( n=='p2_ck_down') ck_p2[ CK_DOWN ] = atoi(v); continue; end
	  if ( n=='p2_ck_menu') ck_p2[ CK_MENU ] = atoi(v); continue; end
	  if ( n=='p2_ck_acti') ck_p2[ CK_ACT_I ] = atoi(v); continue; end
	  if ( n=='p2_ck_actd') ck_p2[ CK_ACT_D ] = atoi(v); continue; end
	  if ( n=='p2_ck_turbo') ck_p2[ CK_TURBO ] = atoi(v); continue; end
	  if ( n=='p2_ck_effect') ck_p2[ CK_EFFECT ] = atoi(v); continue; end
	  if ( n=='p2_ck_volup') ck_p2[ CK_VOLUP ] = atoi(v); continue; end
	  if ( n=='p2_ck_voldn') ck_p2[ CK_VOLDN ] = atoi(v); continue; end
	end;
      end
      fclose( fd );
  end
end

function save_state(string filename)
local
  int fd;
end
begin
  fd = fopen(filename, O_WRITE);
  if( fd )
      fputs( fd, state_unlocked );
      fclose( fd );
  end;
end;

function load_state(string filename)
local
  int fd;
end
begin
  fd = fopen(filename, O_READ);
  if( fd )
      state_unlocked = atoi( fgets(fd) );
      fclose( fd );
  end;
end;

